#include "Status.h"
#include "Vram.h"
#include "Stage.h"
#include "Main.h"
#include "Movable.h"
#include "Actor.h"
#include "Sprite.h"
#include "Print.h"
#include "Chars.h"

static ptr<byte> PrintS(ptr<byte> pVram, ptr<byte> p)
{
    byte c;
    while ((c = *p) != 0) {
        pVram = PrintC(pVram, c);
        ++p;
    }
    return pVram;
}

void PrintStatus() 
{
    PrintS(Vram + 1, "SCORE");
    PrintS(Vram + 7, "HI-SCORE");
    PrintS(Vram + 16, "STAGE");
    PrintByteNumber2(Vram + VramRowSize + 18, CurrentStage + 1);
    PrintS(Vram+ 22, "TIME");

    {
        ptr<byte> pVram;
        pVram = Vram + 27;
        if (RemainCount > 1) {
            byte i;
            i = RemainCount - 1;
            if (i > 2) {
                Put2C(pVram, Char_Remain, 0x47);
                pVram += 2 * VramStep;
                pVram = Put(pVram, Char_Space, 0x47);
                pVram = Put(pVram, Char_Space, 0x47);
                pVram += VramRowSize - 2 * VramStep;
                pVram = Put(pVram, i + 0x10, 0x47);
                pVram = Put(pVram, Char_Space, 0x47);
            }
            else {
                do {
                    Put2C(pVram, Char_Remain, 0x47);
                    pVram += 2 * VramStep;
                    --i;
                } while (i > 0);
            }
        }
    }

    PrintScore();
    PrintTime();
}

void PrintScore()
{
    ptr<byte> pVram;
    
    pVram = PrintNumber5(Vram + VramRowSize, Score);
    PrintC(pVram, '0');

    pVram = PrintNumber5(Vram + VramRowSize + 9, HiScore);
    PrintC(pVram, '0');
}

void PrintTime()
{
    PrintByteNumber3(Vram + VramRowSize + 23, StageTime);
}

static void EraseWindow() 
{
    constexpr byte width = 12;
    constexpr byte height = 3;    
    ptr<byte> pVram;

    pVram = Vram + VramRowSize * 11 + 10;
    repeat (height) {
        repeat (width) {
            pVram = PrintC(pVram, ' ');
        }
        pVram += VramRowSize - width;
    }
}

void PrintGameOver()
{
    EraseWindow();
    PrintS(Vram + VramRowSize * 12 + 11, "GAME OVER");
}

void PrintTimeUp() 
{
    EraseWindow();
    PrintS(Vram + VramRowSize * 12 + 12, "TIME UP");
}

void Title()
{
    static const byte[] TitleBytes = {
        //	A
        0x00, 0xf6, 0xe6, 0xf3, 
        0xe8, 0xf2, 0x00, 0x8e, 
        0xe8, 0xe4, 0xee, 0x8e, 
        0xfa, 0xf8, 0x00, 0xee, 
        //	S
        0xe1, 0xe4, 0xee, 0xe3, 
        0xfa, 0xe3, 0x94, 0xf3, 
        0xe1, 0xf3, 0x00, 0x8e, 
        0x00, 0xee, 0xee, 0xf8, 
        //	C
        0x00, 0xf6, 0xee, 0xe3, 
        0xe8, 0xf2, 0x00, 0x00, 
        0xfa, 0xe3, 0x00, 0x94, 
        0x00, 0xfa, 0xee, 0xf8, 
        //	E
        0xe8, 0xe4, 0xee, 0xf8, 
        0xe8, 0xe3, 0x94, 0xf3, 
        0xe8, 0xf2, 0x00, 0x00, 
        0xfa, 0xee, 0xee, 0xf8, 
        //	N
        0xe8, 0xe3, 0x00, 0x8e, 
        0xe8, 0x8e, 0xe3, 0x8e, 
        0xe8, 0xf2, 0xe6, 0x8e, 
        0xfa, 0xf8, 0x00, 0xee, 
        //	D
        0xe8, 0xe4, 0xe6, 0xf3, 
        0xe8, 0xf2, 0x00, 0x8e, 
        0xe8, 0xf2, 0xe1, 0xe4, 
        0xfa, 0xee, 0xee, 0x00, 
    };

    ClearScreen(); 
    HideAllSprites();
    PrintStatus();
    {
        ptr<byte> p;
        ptr<byte> pVram;
        pVram = Vram + VramRowSize * 7 + 4;
        p = TitleBytes;
        repeat (6) {
            repeat (4) {
                repeat (4) {
                    pVram = Put(pVram, *p, 0x06);
                    ++p;
                }
                pVram += VramRowSize - 4;
            }
            pVram += 4 - VramRowSize * 4;
        }
    }
    PrintS(Vram + VramRowSize * 19 + 9, "PUSH SPACE KEY");
    PrintS(Vram + VramRowSize * 20 + 9, "OR RETURN KEY");
    PrintS(Vram + VramRowSize * 23 + 20, "INUFUTO 2022");
}